﻿namespace A6MinotaurLabyrinth
{
	public static class LabyrinthCreator
	{
		static readonly (int rows, int cols) smallCoords = (4, 4);
		static readonly (int rows, int cols) medCoords = (6, 6);
		static readonly (int rows, int cols) largeCoords = (8, 8);

		// Creates a small 4x4 game.
		public static LabyrinthGame CreateSmallGame()
		{
			(Map map, Location start) = InitializeMap(Size.Small);
			return CreateGame(map, start);
		}

		// Creates a medium 6x6 game.
		public static LabyrinthGame CreateMediumGame()
		{
			(Map map, Location start) = InitializeMap(Size.Medium);
			return CreateGame(map, start);
		}

		// Creates a large 8x8 game.
		public static LabyrinthGame CreateLargeGame()
		{
			(Map map, Location start) = InitializeMap(Size.Large);
			return CreateGame(map, start);
		}
		
		// Helper function that initializes the map size and all the map locations
		// Returns the initialized map and the entrance location (start) so we can
		// set the player starting location accordingly.
		private static (Map, Location) InitializeMap(Size mapSize)
		{
			Map map = mapSize switch
			{
				Size.Small => new Map(smallCoords.rows, smallCoords.cols),
				Size.Medium => new Map(medCoords.rows, medCoords.cols),
				Size.Large => new Map(largeCoords.rows, largeCoords.cols),
			};
			Location start = RandomizeMap(map);
			return (map, start);
		}

		// Creates a map with randomly placed and non-overlapping features
		// The Entrance will be located in a random position along the edge of the map
		// The Sword will be placed randomly in the map but it will not be adjacent to the Entrance
		// Traps & Monsters will be placed randomly
		private static Location RandomizeMap(Map map)
		{
			//TODO: (A6) create an instance of the LabryinthCreatorRng class to get random locations
			Location start = new Location(0, 0);	//TODO: (A6) Randomize the start location of the entrance
			map.SetRoomTypeAtLocation(start, RoomType.Entrance);
			Location swordStart = new Location(2, 2);	//TODO: (A6) Randomize the location of the sword
			map.SetRoomTypeAtLocation(swordStart, RoomType.Sword);
			//TODO: (A7) Randomize the location of the pit rooms
			//add pit room(s) to the map - consider scaling the number of obstacle rooms to the game size.
			return start;
		}

		private static Player InitializePlayer(Location start)
		{
			return new Player(start);
		}

		private static Monster[] InitializeMonsters(Map map)
		{
			//TODO: (A8) Initalize an array of monsters and fill it with one Minotaur instance (and other monster instances).
			//Consider scaling the number of your monster type to the size of the game. However, there should only ever be one minotaur in the labyrinth.
			//Set every new monster's loaction to be random (hint: use the LabryinthCreatorRng class)

			// Ensure monster locations do not overlap existing locations on the map
			return new Monster[] { };
		}

		private static LabyrinthGame CreateGame(Map map, Location start)
		{
			Player player = InitializePlayer(start);
			Monster[] monsters = InitializeMonsters(map);
			return new LabyrinthGame(map, player, monsters);
		}

		private enum Size { Small, Medium, Large };
	}

	//TODO: (A6) Complete this new class that will be used to get random room locations in the labyrinth that are suited for the entrance and the sword
	public class LabryinthCreatorRng
	{
		//TODO: (A6) data fields

		//TODO: (A6) constuctor

		//TODO: (A6) method that returns a random location suited to the entrance
		//the entrance should always be located on the edge of the 2D grid.

		//TODO: (A6) method that returns a random location suited to the sword
		//sword location should not be adjacent or overlapping the start location

		//any additional methods (if needed)

		//TODO: (A7) Create a method that returns a random location suited to the to a Pit room
		//pit room locatoins should only be added to locations that are empty (RoomType.Normal)

		//TODO: (A8) Create a method that returns a random location where it is suitable to add the minotaur monster
		//a minotaur should only be added to a location that is empty (RoomType.Normal)

	
	}

}
